﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ProcessJavaScript
{
    public enum blockTypes { unknown, calc, nonCalc };

    public class JavaScriptExpression
    {
        private string _nodeExe;
        private string _preBlankTmp;
        private string _revBlankTmp;

        public JavaScriptExpression(string mode, string tmpJSFile)
        {
            _nodeExe = @"C:\Program Files\nodejs\node.exe";

            if (mode == "PreBlank")
            {
                _preBlankTmp = tmpJSFile;
            }
            else
            {
                _revBlankTmp = tmpJSFile;
            }
        }

        /*
         * The pre-blanking and reverse blanking operations rely on the
         * existance of the script files which actually do the pre-blanking
         * and reverse blanking, and of course node.exe so we will verify
         * that these exist.
         * 
         * Maybe someone could accidentally delete a script file for
         * example. File.Exists will return false if the caller lacks the
         * required permissions even if path exists, but doing our best here.
         */
        public bool CheckArgs(string mode)
        {
            bool result = true;

            if (!File.Exists(_nodeExe))
            {
                MessageBox.Show(string.Format("  Could not verify/locate the file:\n\n  {0}\n\n  Aborting operation.", _nodeExe), "CheckArgs(): File Not Found", MessageBoxButtons.OK, MessageBoxIcon.Error);

                result = false;
            }
            else
            {
                if (mode == "PreBlank")
                {
                    if (!File.Exists(_preBlankTmp))
                    {
                        MessageBox.Show(string.Format("  Could not verify/locate the file:\n\n  {0}\n\n  Aborting operation.", _preBlankTmp), "CheckArgs(): File Not Found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        result = false;
                    }
                }
                else
                {
                    if (!File.Exists(_revBlankTmp))
                    {
                        MessageBox.Show(string.Format("  Could not verify/locate the file:\n\n  {0}\n\n  Aborting operation.", _revBlankTmp), "CheckArgs(): File Not Found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        result = false;
                    }
                }
            }

            return result;
        }

        /*
         * Called From: PrepareToBlank()
         * 
         * preBlankCode - Contains the code that we will perform the pre-blanking on.
         */
        public bool DoPreBlank(string preBlankCode)
        {
            string output = "";
            bool result = false;

            try
            {
                Process p = new Process();

                p.StartInfo.UseShellExecute = false;
                p.StartInfo.RedirectStandardOutput = true;
                p.StartInfo.CreateNoWindow = true;
                p.StartInfo.FileName = _nodeExe;

                /*
                 * The pre-blank application, _preBlankTmp, local copy on user's
                 * computer, in their temp folder, and the file that will be worked
                 * on, preBlankCode, which will be overwritten with the results.
                 */
                p.StartInfo.Arguments = _preBlankTmp + " " + preBlankCode;

                if (p.Start() == true)
                {
                    // To avoid deadlocks, always read the output stream first and then wait.
                    output = p.StandardOutput.ReadToEnd();

                    p.WaitForExit();

                    if (output == "OK")
                    {
                        result = true;
                    }
                    else
                    {
                        MessageBox.Show(string.Format("  {0}\n\n   Aborting Operation.", output), "jsExpr.DoPreBlank() ERROR:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                result = false;
            }

            return result;
        }

        /*
         * Called From: ReverseBlank()
         * 
         * revBlankCode - Contains the code that we will perform the reverse blanking on.
         */
        public bool DoRevBlank(string revBlankCode)
        {
            string output = "";
            bool result = false;

            try
            {
                Process p = new Process();

                p.StartInfo.UseShellExecute = false;
                p.StartInfo.RedirectStandardOutput = true;
                p.StartInfo.CreateNoWindow = true;
                p.StartInfo.FileName = _nodeExe;

                /*
                 * The reverse blank application, _revBlankTmp, local copy on user's
                 * computer, in their temp folder, and the file that will be worked
                 * on, revBlankCode, which will be overwritten with the results.
                 */
                p.StartInfo.Arguments = _revBlankTmp + " " + revBlankCode;

                if (p.Start() == true)
                {
                    // To avoid deadlocks, always read the output stream first and then wait.
                    output = p.StandardOutput.ReadToEnd();

                    p.WaitForExit();

                    if (output == "OK")
                    {
                        result = true;
                    }
                    else
                    {
                        MessageBox.Show(string.Format("  {0}\n\n   Aborting Operation.", output), "jsExpr.DoRevBlank() ERROR:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                result = false;
            }

            return result;
        }
    }

    public class CodeBlock
    {
        public blockTypes blockType = blockTypes.unknown;
        public List<string> codeLine = new List<string>();
        public List<int> codeIndex = new List<int>();
        public string calcBox = null;
        public bool hasVar = false;
    }
}